/* gui.c - part of clavier
 *
 * Copyright (C) 1998 Simon Kgedal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 */

#include <config.h>
#include <stdio.h>
#include <getopt.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "progchange.h"
#include "noteinput.h"
#include "midi.h"
#include "midievents.h"
#include "out.h"
#include "gui.h"

/* from main.c */
extern Output *output;

static GtkWidget *clavier;
static GtkWidget *noteinput;
static GtkWidget *scale;
static GtkWidget *spinner;

MidiHandle *midihandle = NULL;

/* handles finished MIDI messages
 */

static void
my_midi_handler (MidiHandle *handle)
{
  Clavier *clavier;

  g_return_if_fail (handle->extradata != NULL);
  g_return_if_fail (IS_CLAVIER (handle->extradata));

  clavier = CLAVIER (handle->extradata);

  switch (midi_get_status (handle))
    {
    case MIDI_NOTE_ON:
      clavier_press (clavier, midi_get_note (handle));
      break;

    case MIDI_NOTE_OFF:
      clavier_release (clavier, midi_get_note (handle));
      break;
    }
}

void
gui_midi_handler_init (void)
{
  midihandle = midi_handle_new (my_midi_handler);
  midihandle->extradata = (void*) clavier;
}

/* events
 */

static gint
delete_event(GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
  return (FALSE);
}

/* helper for `clavierkey_press_event' and `note_input_press_event'
 */

static void
press (gint key)
{
  gint channel = gtk_spin_button_get_value_as_int 
    (GTK_SPIN_BUTTON (spinner)) - 1;

  GtkAdjustment *adj = gtk_range_get_adjustment (GTK_RANGE (scale));
  gint pressure = (GTK_ADJUSTMENT (adj)->value);
  
  output_noteon (output, channel, key, pressure);

  clavier_press (CLAVIER (clavier), key);
}

/* helper for `clavierkey_release_event' and `note_input_release_event'
 */

static void
release (gint key)
{
  gint channel = gtk_spin_button_get_value_as_int 
    (GTK_SPIN_BUTTON (spinner)) - 1;

  output_noteoff(output, channel, key);

  clavier_release (CLAVIER (clavier), key);
}

/* received when a key on the "clavier" is pressed
 */

static gint
clavierkey_press_event (GtkWidget *widget, gint key, gpointer data)
{
  press (key);
  return FALSE;
}

/* received when a key on the "clavier" is released
 */

static gint
clavierkey_release_event (GtkWidget *widget, gint key, gpointer data)
{
  release (key);
  return FALSE;
}

/* received when a key on the keyboard is pressed (through NoteInput)
 */

static gint
note_input_press_event (GtkWidget *widget, gint key, gpointer data)
{
  press (key);
  return FALSE;
}

/* received when a key on the keyboard is released (through NoteInput) */

static gint
note_input_release_event (GtkWidget *widget, gint key, gpointer data)
{
  release (key);
  return FALSE;
}

/* received when the transpose base is changed */

static gint
note_input_transpose_change_event (GtkWidget *widget, gint key, 
				   gpointer data)
{
  clavier_set_transpose_base (CLAVIER (clavier), key);
  return FALSE;
}

/* when `About...' is clicked
 * I might do something a *bit* more beautiful :)
 */

static void
about_response (gpointer data)
{
  GtkWidget *window;
  GtkWidget *button;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "About clavier");

  button =  gtk_button_new_with_label("Clavier " 
				      VERSION " by Simon Kgedal 1998");
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));

  gtk_container_add (GTK_CONTAINER (window), button);

  gtk_widget_show (button);
  gtk_widget_show (window);
}

/* When `Send -> Program Change' is clicked */

static void
progchange_response (gpointer data)
{
  progchange_show (spinner);
}

/* When `Send -> All Notes Off' is clicked */

static void
allnotesoff_response (gpointer data)
{
  gint channel = gtk_spin_button_get_value_as_int 
    (GTK_SPIN_BUTTON (spinner)) - 1;

  output_all_notes_off (output, channel);
}

/* make `Program' menu
 */

static GtkWidget *
gui_make_program_menu ()
{
  GtkWidget *root_menu;
  GtkWidget *menu;
  GtkWidget *menu_item;

  menu = gtk_menu_new ();

  menu_item = gtk_menu_item_new_with_label ("About...");
  gtk_menu_append (GTK_MENU (menu), menu_item);
  gtk_signal_connect_object (GTK_OBJECT (menu_item), "activate",
			     GTK_SIGNAL_FUNC (about_response), NULL);
  gtk_widget_show (menu_item);

  /* TODO: separator */

  menu_item = gtk_menu_item_new_with_label ("Quit");
  gtk_menu_append (GTK_MENU (menu), menu_item);
  gtk_signal_connect_object (GTK_OBJECT (menu_item), "activate",
			     GTK_SIGNAL_FUNC (delete_event), NULL);
  gtk_widget_show (menu_item);

  /* "root menu" */

  root_menu = gtk_menu_item_new_with_label ("Program");
  gtk_widget_show (root_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu), menu);

  return (root_menu);
}

/* make `Send' menu
 */

static GtkWidget *
gui_make_send_menu ()
{
  GtkWidget *root_menu;
  GtkWidget *menu;
  GtkWidget *menu_item;

  menu = gtk_menu_new ();

  menu_item = gtk_menu_item_new_with_label 
    ("Program Change");
  gtk_menu_append (GTK_MENU (menu), menu_item);
  gtk_signal_connect_object (GTK_OBJECT (menu_item), "activate",
			     GTK_SIGNAL_FUNC (progchange_response), NULL);
  gtk_widget_show (menu_item);

  menu_item = gtk_menu_item_new_with_label ("All Notes Off");
  gtk_menu_append (GTK_MENU (menu), menu_item);
  gtk_signal_connect_object (GTK_OBJECT (menu_item), "activate",
			     GTK_SIGNAL_FUNC (allnotesoff_response), NULL);
  gtk_widget_show (menu_item);

  root_menu = gtk_menu_item_new_with_label ("Send");
  gtk_widget_show (root_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu), menu);

  return (root_menu);
}

/* create GUI
 */

void
gui_init (ClavierDir clavierdir, ClavierType claviertype,
	  gint start, gint end)
{
  GtkWidget *window;
  GtkWidget *menu_bar;
  GtkWidget *mainvbox;
  GtkWidget *box1, *box2;
  GtkWidget *label;
  GtkWidget *frame;
  GtkObject *adjustment;

  gint keywidth;

  GString *config_file = NULL;

  /* main window 
   */

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title (GTK_WINDOW (window), "clavier");

  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);

  /* main vbox that holds 1) the menu 2) the rest of the app  */

  mainvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (mainvbox);
  gtk_container_add (GTK_CONTAINER (window), mainvbox);

  /* menu */

  menu_bar = gtk_menu_bar_new ();
  gtk_box_pack_start (GTK_BOX (mainvbox), menu_bar, FALSE, FALSE, 2);
  gtk_widget_show (menu_bar);
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), gui_make_program_menu ());
  gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), gui_make_send_menu ());

  /* note input that receives keyboard press/release events
   */

  noteinput = note_input_new (); 

  gtk_signal_connect (GTK_OBJECT (noteinput), "note_input_press",
		      GTK_SIGNAL_FUNC (note_input_press_event),
		      NULL);

  gtk_signal_connect (GTK_OBJECT (noteinput), "note_input_release",
		      GTK_SIGNAL_FUNC (note_input_release_event),
		      NULL);

  gtk_signal_connect (GTK_OBJECT (noteinput), "note_input_transpose_change",
		      GTK_SIGNAL_FUNC (note_input_transpose_change_event),
		      NULL);

  /*  gtk_container_add (GTK_CONTAINER (window), noteinput); */
  gtk_box_pack_end (GTK_BOX (mainvbox), noteinput, FALSE, FALSE, 2);

  gtk_widget_show (noteinput);

  if (clavierdir == CLAVIER_DIR_HORIZONTAL)
    {
      /* vertical box
       */

      box1 = gtk_vbox_new (FALSE, 0);
    }
  else
    {
      /* horizontal box
       */

      box1 = gtk_hbox_new (FALSE, 0);
    }

  gtk_container_border_width (GTK_CONTAINER (box1), 10);
  gtk_box_set_spacing (GTK_BOX(box1), 5);

  gtk_container_add (GTK_CONTAINER (noteinput), box1);

  /* frame to keep clavier in
   */

  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
  gtk_box_pack_start (GTK_BOX (box1), frame, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame), 0);
  gtk_widget_show (frame);

  /* clavier
   */

  clavier = clavier_new ();

  if (claviertype == CLAVIER_TYPE_NORMAL)
    keywidth = 8;
  else
    keywidth = 10;

  clavier_set_range (CLAVIER(clavier), start, end);

  if (clavierdir == CLAVIER_DIR_VERTICAL)
    {
      gtk_drawing_area_size (GTK_DRAWING_AREA (clavier), 60, 
			     (end-start+1) * keywidth);
      clavier_set_clavier_dir (CLAVIER(clavier), CLAVIER_DIR_VERTICAL);
    }
  else
    {
      gtk_drawing_area_size (GTK_DRAWING_AREA (clavier), 
			     (end-start+1) * keywidth, 60);
      clavier_set_clavier_dir (CLAVIER (clavier), CLAVIER_DIR_HORIZONTAL);
    }

  clavier_set_clavier_type (CLAVIER (clavier), claviertype);
  clavier_set_show_transpose (CLAVIER (clavier), TRUE);

  gtk_signal_connect (GTK_OBJECT (clavier), "clavierkey_press",
		      GTK_SIGNAL_FUNC (clavierkey_press_event),	
		      NULL);

  gtk_signal_connect (GTK_OBJECT (clavier), "clavierkey_release",
		      GTK_SIGNAL_FUNC (clavierkey_release_event),
		      NULL);

  gtk_container_add (GTK_CONTAINER (frame), clavier);
  gtk_widget_show (clavier);

  /* box containing velocity scale and channel spinner
   */

  if (clavierdir == CLAVIER_DIR_HORIZONTAL)
    {
      box2 = gtk_hbox_new (FALSE, 0);
    }
  else
    {
      box2 = gtk_vbox_new (FALSE, 0);
    }

  gtk_box_pack_start (GTK_BOX (box1), box2,
		      FALSE, FALSE, 0);

  /* scale
   */

  if (clavierdir == CLAVIER_DIR_HORIZONTAL)
    {
      adjustment = gtk_adjustment_new (60.0, 0.0, 127.0, 1.0, 10.0, 0.0);
      scale = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
    }
  else
    {
      adjustment = gtk_adjustment_new (60.0, 0.0, 127.0, -1.0, -10.0, 0.0);
      scale = gtk_vscale_new (GTK_ADJUSTMENT (adjustment));
    }      

  gtk_scale_set_digits (GTK_SCALE (scale), 0);

  /* show values of the scale */

  gtk_scale_set_draw_value (GTK_SCALE (scale), TRUE);

  gtk_box_pack_start (GTK_BOX (box2), scale, TRUE, TRUE, 0);

  gtk_widget_show (scale);

  /* MIDI channel spinner (first label for it) */

  label = gtk_label_new ("Channel: ");
  gtk_box_pack_start (GTK_BOX (box2), label,
		    FALSE, FALSE, 10);
  gtk_widget_show (label);

  adjustment = gtk_adjustment_new (1.0, 1.0, 16.0, 1.0, 5.0, 0.0);

  spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1.0, 0);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinner),
				     GTK_UPDATE_ALWAYS);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinner), TRUE);

  gtk_box_pack_start (GTK_BOX (box2), spinner,
		      FALSE, FALSE, 0);

  gtk_widget_show (spinner);

  /* focus to note input so it gets key events  */

  gtk_window_set_focus (GTK_WINDOW (window), noteinput);

  /* show stuff
   */

  gtk_widget_show (box2);
  gtk_widget_show (box1);
  gtk_widget_show (window);

  /* read config file - we don't care about return code, we'll just
   * not have any keyboard input if it fails
   */

  config_file = g_string_new (g_get_home_dir ());
  config_file = g_string_append (config_file, "/.noteinput");
  note_input_read_config (NOTE_INPUT (noteinput), config_file->str);
  g_string_free (config_file, TRUE);

  /* give transpose base to clavier */

  clavier_set_transpose_base (CLAVIER (clavier),
			      note_input_get_transpose_base
			      (NOTE_INPUT (noteinput)));
}

void
gui_destroy (void)
{
  if (midihandle)
    {
      midi_handle_free (midihandle);
    }
}
