/* clavier.c - part of clavier
 *
 * Copyright (C) 1998 Simon Kgedal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License in the file COPYING for more details.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <getopt.h>
#include <string.h>
#include "gui.h"
#include "input.h"
#include "out.h"
#include "clavier.h"
#include "glib_1_0_compat.h"

Output *output;

/* shall we read input? (from a fifo or stdin) */
static int do_input = FALSE;

/* MIDI-key range */
static gint start = 24;
static gint end = 103;

static ClavierType claviertype = CLAVIER_TYPE_NORMAL;
static ClavierDir clavierdir = CLAVIER_DIR_HORIZONTAL;

/* print usage information and exit
 */

static void
usage_exit (int ret_code)
{
  printf ("Usage: %s [OPTION]...\n\n"
	  "  -t n|s, --type=n\t\tset type to normal|sequencer\n"
	  "  -d h|v, --dir=v\t\tset dir to horizontal|vertical\n"
	  "  -s N, --start=N\t\tkey interval start\n"
	  "  -e N, --end=N\t\tkey interval end\n"
	  "  -f file, --fifo=file\t\tcreates a fifo and reads from it\n"
	  "  -o file, --output=file\toutputs to this file\n"
	  "  -h, --help\t\t\tdisplay this help and exit\n"
	  "  -v, --version\t\t\toutput version information and exit\n",
	  g_get_prgname());

  exit (ret_code);
}

/* parse the options on command line
 */

static int
parse_options (int argc, char *argv [])
{
  int c, option_index = 0;

  /* shall we do any output? */
  int do_output = FALSE;

  char *fifo_filename = NULL;
  char *output_filename = NULL;

  gboolean err = FALSE;

  struct option opts[] =
  {
    { "dir", required_argument, NULL, 'd' },
    { "type", required_argument, NULL, 't' },
    { "start", required_argument, NULL, 's' },
    { "end", required_argument, NULL, 'e' },
    { "fifo", required_argument, NULL, 'f' },
    { "output", required_argument, NULL, 'o' },
    { "help", no_argument, NULL, 'h' },
    { "version", no_argument, NULL, 'v' },
    { 0, 0, 0, 0 }
  };

  g_set_prgname (argv[0]);

  output = output_new ();

  gtk_init (&argc, &argv);

  /* parse options
   */

  while (1)
    {
      c = getopt_long (argc, argv, "d:t:s:e:f:o:hv", opts, 
		       &option_index);

      if (c == -1)
	break;

      switch (c)
	{
	case 'd':
	  if (*optarg == 'v')
	    clavierdir = CLAVIER_DIR_VERTICAL;
	  break;

	case 't':
	  if (*optarg == 's')
	    claviertype = CLAVIER_TYPE_SEQUENCER;
	  break;

	case 's':
	  /* should complain if out of range but i'm lazy */
	  start = CLAMP (atoi (optarg), 0, 127);
	  break;

	case 'e':
	  /* should complain if out of range but i'm lazy */
	  end = CLAMP (atoi (optarg), 0, 127);
	  break;

	case 'f':
	  fifo_filename = g_strdup (optarg);
	  do_input = TRUE;
	  break;

	case 'o':
	  output_filename = g_strdup (optarg);
	  do_output = TRUE;
	  break;

	case 'h':
	  usage_exit (0);

	case 'v':
	  fputs ("clavier " VERSION "\n", stdout);
	  exit (0);

	case '?':
	  /* `getopt_long' already printed an error message. */
	  fprintf (stderr, "Try `%s --help' for more information.\n",
		   g_get_prgname ());
	  exit (1);

	default:
	  abort ();
	}
    }

  /* should complain but i'm lazy */
  if (end < start)
    end = start;

  /* do input setup? 
   */

  if (do_input)
    {
    input_init (fifo_filename);
    g_free (fifo_filename);
    }

  /* output setup
   */

  if (do_output)
    {
      if (output_open (output, output_filename))
	err = TRUE;

      g_free (output_filename);
    }

  return (!err);
}

/* main */

int
main(int argc, char *argv [])
{
  if (!parse_options (argc, argv))
    {
      return (1);
    }

  gui_init (clavierdir, claviertype, start, end);
  
  gtk_main ();

  gui_destroy ();
      
  if (do_input)
    {
      input_exit ();
    }

  return (0);
}

